.module template

max_entities == 20

.macro DefineEntity _name, _type, _cmps, _x, _y, _w, _h, _vx, _vy, _orientation, _sprite, _animation, enemy_type, ai_state, _ai_behaviour
    _name::
        .db _type, _cmps
        .db _x, _x, _x, _y, _y, _y
        .db _w, _h 
        .db _vx, _vy
        .db _orientation
        .dw _sprite, _animation
        .db default_animation_frames
        .db enemy_type, ai_state
        .dw _ai_behaviour
.endm

;; #####################################################################################
;; #                                ENTITY STRUCTURE                                         
;; #####################################################################################
;; #     ATTRIBUTE      #                    #  BYTE  #       # DESCRIPTION #     
;; #####################################################################################
        e_type                  ==              0
        e_cmps                  ==              1
        e_x0                    ==              2               ;;Position this frame
        e_x1                    ==              3               ;;Position 1 frame ago
        e_x2                    ==              4               ;;Position 2 frames ago
        e_y0                    ==              5
        e_y1                    ==              6
        e_y2                    ==              7
        e_w                     ==              8
        e_h                     ==              9
        e_vx                    ==              10
        e_vy                    ==              11
        e_orientation           ==              12
        e_sprite_l              ==              13
        e_sprite_h              ==              14
        e_animated_l            ==              15
        e_animated_h            ==              16
        e_frames_animation      ==              17
        e_enemy_type            ==              18
        e_estado_ia             ==              19
        e_ai_behaviour_l        ==              20
        e_ai_behaviour_h        ==              21
;; #########################################################################################################
        entity_size             ==              22  ;; (should be the same as the number of lines above)
;; #########################################################################################################


;; ENTITY TYPES (masks)
t_invalid       ==    0x01
t_player        ==    0x02
t_enemy         ==    0x04 
t_bala          ==    0x08
t_bala_enemy    ==    0x10      
t_invisible     ==    0x20
t_default       ==    0x40 
t_toBeKilled    ==    0x80

;; ENEMY TYPES (not masks!)
et_pato                     ==  0x01
et_pato_small               ==  0x02
et_tempo                    ==  0x04
et_reflective_pato_big      ==  3
et_reflective_pato_small    ==  5
et_fantasma                 ==  0x08
et_tanque                   ==  0x10

;; ENTITY COMPONENTS
c_render    ==  0x01
c_movable   ==  0x02 
c_input     ==  0x04 
c_ai        ==  0x08 
c_animated  ==  0x10 
c_default   ==  0x20 

;; GENERAL CONSTANTS
nullptr                     ==      0x0000
default_animation_frames    ==      0
default_just_created        ==      1       ;;  1 or 0

;; ORIENTATIONS
o_up     ==     1      
o_down   ==     2  
o_right  ==     3
o_left   ==     4

;; BULLET CONSTANTS
bala_width_vertical    ==   1
bala_height_vertical   ==   4    
bala_width_horizontal  ==   1    
bala_height_horizontal ==   2


;; Constants shot(bala) speeds
vel_bala_u    ==    -12
vel_bala_d    ==    12
vel_bala_r    ==    4
vel_bala_l    ==    -4

;; Constants shot(bala_enemy) speeds
vel_balae_u    == #-6
vel_balae_d    == #6
vel_balae_r    == #2
vel_balae_l    == #-2


;;;; PLAYER
;; Constants player speeds
vel_player_u    ==  -4
vel_player_d    ==  4
vel_player_r    ==  1
vel_player_l    ==  -1

;; Size of sprite player
player_width        ==  4
player_height       ==  10
player_width_medio  ==  player_width / 2
player_height_medio ==  player_height / 2
margen              ==  2

;; TEMPO (enemy wizard) constants
default_ai_state_tempo  ==      01 ;; alias for default_state
default_state           ==      01
speed_slow              ==      08
speed_standard          ==      03
speed_fast              ==      01
tempo_charge_time_slow  ==      12
tempo_charge_time_fast  ==      72
vidas_tempo             ==      03

;; BASIC_PATO states
;; states will actually be used to manage the pato's speed directly
basic_pato_down_state   ==      1
basic_pato_up_state     ==      2
sinusoidal_pato         ==      18

;; FANTASMA constants
tiempo_charge_time_normal == #40
tiempo_movement_time_normal == #2
tiempo_charge_time_transparente == #36
tiempo_movement_time_transparente == #6
estado_opaco == #2
estado_transparente == #3

;; SMALL PATO

small_fast_pato_default_lifes   ==      2

;; Points earned by killing each enemy
punctuation_pato == #0x10
punctuation_pato_small == #0x05
punctuation_fantasma == #0x20
punctuation_reflective_pato == #0x110
punctuation_tempo == #0x40
punctuation_tanque == #0x18
punctuation_losing_life == #0x1500


;; Tanque
time_tanque_shoot == 5

;; SCREEN CONSTANTS
screen_width    ==      76
screen_height   ==      184
screen_start    ==      0xC000

;; OLEADAS
rd_corner       ==      0xC6D8 ;;right-down
ru_corner       ==      0xC8E8 
ld_corner       ==      0xC694 
lu_corner       ==      0xC8A4 
wait_time_between_oleadas == 0x60

;; MUERTE animation
time_animation == 10
inmunidad_time == 20

;; GLOBAL DEFINITIONS
.globl bala_lettuce
.globl bala_enemy
.globl pato_entidad
.globl taco_entidad
.globl pato_small_entidad
.globl tempo_entidad
.globl reflective_pato_big_entity
.globl reflective_pato_small_entity
.globl tempo_vidas_restantes
.globl small_fast_pato_lifes
.globl tanque_entidad
.globl fantasma_entidad
.globl get_random
.globl popping_entidad
