.module entity

.include "cpctelera.h.s"
.include "entity.h.s"
.include "man/collision.h.s"
.include "man/animation.h.s"
.include "man/mania.h.s"
.include "man/game.h.s"
.include "assets/template.h.s"
.include "assets/sprites.h.s"
.include "sys/render.h.s"

;;_num_entities:: .db 0
;;_last_elem_ptr:: .dw _entity_array ;;Memory address where the next new entity will be written
_entity_array::
    .ds max_entities * entity_size 
posicion_libre: ;; Para indicar que se ha encontrado una posicion libre en la que insertar una nueva entidad, guarrada pero se aprovecha el forall
	.dw #0

vidas_taco::
	.db #1

new_bala:: .db  #1 ;; Se puede utilizar como contador para no crear balas
;; En este caso será un poco distinto el contador de balas para el enemigo.
;; Ya que pueden haber muchas entidades del mismo enemigo a la vez, tenemos que considerar que pueden haber
;; tantas balas en el mapa a la vez como entidades de tipo tanque. Por ello al crear una entidad tipo tanque le añadiremos
;; uno a new_bala_enemy para que sepamos cuantas balas podemos crear. Cuando creemos una le descontaremos uno al contador
;; y cuando se elimine una, le sumaremos uno al contador.
new_bala_enemy:: .db  #1
new_bomba:: .db #1 ;; Mismo uso que bala. TODO:: Que solo se pueda usar cuando mates a un enemigo.
tiempo_bala:: .db #0x0A

entity_man_init::

	;; Llama a otros managers, como collisions, IA
	call man_collision_init
	call man_animation_init
	call man_ia_init

	ld hl, #man_entity_invalida
	call man_entity_forall_incluso_invalidas

    ret

;; Invalida una entidad
;; INPUT IX: entidad a invalidar
man_entity_invalida:
	;;ld__d_ixh
	;;ld__e_ixl
	;;xor a
	;;ld bc, #entity_size
    ;;call cpct_memset_asm
	ld e_type(ix), #t_invalid
	ret



;; Destruye ix, a
;; Devuelve: - ix puntero al array de entidades, 
;;			 - a numero de entidades
man_entity_getArray::
	ld ix, #_entity_array
	ret

;; Mira si la posicion en ix es invalida
;; INPUT IX: entidad a checkear
man_entity_check_invalid:
	ld a, (posicion_libre)
	or a
	ret nz ;; Si no es 0 ya se ha encontrado una 
	ld a, e_type(ix) 
	and #t_invalid
	ret z ;; No es inválido, se sigue biscando
	push hl
	ld hl, #posicion_libre
	ex de, hl
	ld__d_ixh ;; hl = ix (cambiado con de)
	ld__e_ixl
	ex de, hl
	ld (posicion_libre), hl
	pop hl

	ld e_type(ix), #0
	ret


;; Busca la primera entidad disponible para crear una nueva
;; Devuelve HL: puntero al elemento añadido
man_entity_find_first::
	ld hl, #posicion_libre
	ld (hl), #0x0000

    ld hl, #man_entity_check_invalid
    call man_entity_forall_incluso_invalidas
	ld hl, (posicion_libre)
    ret

;; INPUT: HL memory address pointing to entity_size bytes of information
;; Saves IX in the stack
man_entity_create_save_ix::
	push ix 
	call man_entity_create
	pop ix
	ret 

;; Crea nueva entidad con animaccion popping que creará a la entidad de verdad
;; INPUT: HL: puntero con los valores a inicializar
man_create_popping::
	ld ix, #popping_entidad
	ld e_enemy_type(ix), l 
	ld e_estado_ia(ix), h 
	inc hl 
	inc hl 
	ld a, (hl)
	ld e_x0(ix), a 
	inc hl 
	ld a, (hl)
	ld e_x1(ix), a 
	inc hl 
	ld a, (hl)
	ld e_x2(ix), a 
	inc hl 
	ld a, (hl)
	ld e_y0(ix), a 
	inc hl 
	ld a, (hl)
	ld e_y1(ix), a 
	inc hl 
	ld a, (hl)
	ld e_y2(ix), a 

	ld e_cmps(ix), #c_animated|c_ai
	ld hl, #popping_entidad 
	call man_entity_create
	ret

;; Crea nueva entidad y le inicializa valores
;; INPUT: HL: puntero con los valores a inicializar
;; DESTROY: F, BC, HL, DE
;; 2bytes de stack
;; devuelve ix puntero al componente creado
man_entity_create::
	push hl
	call man_entity_find_first

	ex de, hl
	;; IX = DE, macro de cpctelera porque la instruccion no existe
	ld__ixh_d ;; ix = DE
	ld__ixl_e 
	ld bc, #entity_size
	;; Copia pega que ya habíamos visto
	;; DE nueva entidad				
	;; BC el counter
	pop hl
	ldir

	ld a, #t_enemy
    and e_type(ix)
	jp z, no_es_enemy
	ld hl, #level_enemies ;; Añadir el counter del nivel
	inc (hl)
no_es_enemy:
	;; Mira componentes para añadir managers al objeto
	ld a, e_cmps(ix)
	and #c_movable					  ;; Si tiene el componente movable para colisiones, se hará lo mismo con ia y demás
	call nz, man_collision_add
	;; No se pueden concatenar porque el and se carga el valor de a

	ld a, e_cmps(ix)
	and #c_animated					  
	call nz, man_animation_add

	ld a, e_cmps(ix)
	and #c_ai					  
	call nz, man_ia_add
	ret


;;INPUT: ix entity to be deleted (bit a invalido)
man_entity_delete: ;; No intercambia
	ld e_type(ix), #t_invalid
	call sys_render_clean_entity
	ret


;; INPUT: Tenemos en ix la entidad.
;; Entity will be deleted if has flag to_be_deleted
man_entity_clean:
	ld a, #t_toBeKilled
	and e_type(ix)
	ret z ;; Si el and vale 0 que implicaría que no es una entidad a borrar, saldría de la función.
	;; Recargamos el contador para crear una nueva bala
	ld a, e_type(ix)
	and #t_bala
	jr z, borrar_bala_enemy
	borrar_bala:
		ld hl, #new_bala
		ld (hl), #1
		jr llamar_funcion_borrado
	
	borrar_bala_enemy:
		ld a, e_type(ix)
		and #t_bala_enemy
		jr z, llamar_funcion_borrado
		;;ld hl, #new_bala_enemy
		;;ld (hl), #1
		ld a, e_animated_l(ix)
    	inc a
    	ld e_animated_l(ix), a


	llamar_funcion_borrado:
		call man_entity_delete
	terminar:
		ret

;; Deletes all entities marked with to_be_killed flag. Cannot use forall because array's size changes with
;; each iteration
man_entities_clean::
	ld b, #max_entities
	inc b ;; Counter
	ld ix, #_entity_array
	ld de, #entity_size

	man_entities_clean_loop:
		dec b
		ret z
		ld a, #max_entities ;; no cambiará
		ex af, af'
		exx
		call man_entity_clean
		exx
		ex af, af'
		cp #max_entities
		jr nz, man_entities_clean_loop
		add ix, de
		jr man_entities_clean_loop


_registro_hl_subrutina: .dw 0x0000

;; Es invalida una entidad?
;; INPUT IX: entidad a invalidar
;; DEVUELVE bit de z (1 si es invalida)
man_is_entity_invalid::
	ld a, #t_invalid
	and e_type(ix)
	ret

;; Por todas las entidades del array (que no son invalidas)
;; INPUT: hl dirección de memoria de la función
;; A PRIMA NO SE TOCA
man_entity_forall::
	ld a, #max_entities ;; Iterar por todo el array, el counter
	call man_entity_getArray;; Asigna el array en ix 
	ld (_registro_hl_subrutina), hl
bucle:
	ex af, af' ;; guarda af con las primas
	call man_is_entity_invalid
	jp nz, continuacion ;; no esta guardada la entidad
	ld bc, #continuacion
	push bc
	ld hl, (_registro_hl_subrutina)
	jp (hl) 
continuacion:
	ld de, #entity_size ;; Se accede a la siguiente posicion del array
    add ix, de
	
	ex af, af' ;; lo sacamos de las primas
	dec a
	jp nz, bucle
	ret

;; Igual que el otro forall pero por todas todas, sin saltarse invalidas. Sobre todo para gestionar
man_entity_forall_incluso_invalidas:
	ld a, #max_entities ;; Iterar por todo el array, el counter
	call man_entity_getArray;; Asigna el array en ix 
	ld (_registro_hl_subrutina), hl
bucle2:
	ex af, af' ;; guarda af con las primas

	ld bc, #continuacion2
	push bc
	ld hl, (_registro_hl_subrutina)
	jp (hl) 
continuacion2:
	ld de, #entity_size ;; Se accede a la siguiente posicion del array
    add ix, de
	
	ex af, af' ;; lo sacamos de las primas
	dec a
	jp nz, bucle2
	ret
