.module game

.include "cpctelera.h.s"
.include "game.h.s"
.include "man/entity.h.s"
.include "man/collision.h.s"
.include "man/animation.h.s"
.include "man/vecentity.h.s"
.include "man/lvl.h.s"
.include "man/music.h.s"
.include "sys/physics.h.s"
.include "sys/render.h.s"
.include "sys/input.h.s"
.include "sys/ai.h.s"
.include "sys/syscol.h.s"
.include "sys/sysanimation.h.s"
.include "assets/sprites.h.s"
.include "assets/template.h.s"

.globl cpct_drawStringM0_asm
.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm
.globl cpct_drawSolidBox_asm
.globl cpct_drawStringM1_asm
.globl cpct_drawStringM1_f_asm
.globl cpct_memset_asm
.globl cpct_setDrawCharM1_asm
.globl cpct_getScreenPtr_asm
.globl cpct_isAnyKeyPressed_f_asm
.globl cpct_drawSprite_asm


user_points::
	.dw #0x0000

speed_of_game:: ;; Prince of Persia reference, enemy that will use it
	.db #03

level_enemies:: ;; Counter de los enemigos por nivel
	.db #00

actual_lvl:: ;;Lvl the user is in
	.db #01

counter_punctuation: ;; Number of halts to decrease punctuation
	.db #update_punctuation_wait 

survive_mode_activated::
	.db #00

inmunidad_counter::
	.db #00

end_game_timeout    = 20
init_time_waiting   = 5
update_punctuation_wait = 8

gameover_text:
	.asciz "GAME OVER"

score_text:
	.asciz "SCORE:"

wingame_text:
	.asciz "YOU WIN"

tacos_text:
	.asciz "TACO"

vs_text:
	.asciz "VS"

patos_text:
	.asciz " PATOS"

text_start_and_pause:
	.asciz "SPACE to start / K to pause"

text_mode_adventure:
	.asciz "PLAY ADVENTURE MODE"

text_mode_survival:
	.asciz "PLAY SURVIVAL MODE"

text_mute:
	.asciz "M to mute"

;; El modo escogido siendo el 0(default) el adventure y el 1 el survival
chosen_mode:
	.db #00

big_screen_palette:
	.db	HW_BLACK, HW_BRIGHT_WHITE, HW_PASTEL_YELLOW, HW_RED

;; INPUT d: Frames to wait
;; INPUT e: Dont play music (1) or do play music (any other value)
;; PLAYS MUSIC
;; DESTORYS : AF BC
wait_halts::
   	call cpct_waitVSYNC_asm

	halt
	halt
	halt

	dec d
	jp nz, wait_halts
	ret 

man_game_init_lvl_common:
	call sys_physics_init
	call sys_input_init
	call sys_render_init
	call entity_man_init
	call sys_collision_init
	call sys_animation_init
	call sys_ai_init

	ld hl, #level_enemies
	ld (hl), #0

	ld hl, #taco_entidad
	call man_entity_create 

   	ld d, #init_time_waiting
	call wait_halts

	ld hl, #main_palette
	ld de, #4
	call cpct_setPalette_asm
	ld hl, #speed_of_game
    ld (hl), #speed_standard
	ld hl, #new_bala
	ld (hl), #1
	ret

;; Calls to game init for a new level
man_game_init_lvl::
	call man_game_init_lvl_common
	draw_vidas_level_twice
	ret

;; Calls to game init for a new level
man_game_init_lvl_surv:
	call man_game_init_lvl_common
	ret

;; Calls system and manager init functions. Inits music and char colors
man_game_init:

	ld hl, #user_points
    ld (hl), #0x00
	inc hl 
	ld (hl), #0x00

	ld hl, #actual_lvl
	ld (hl), #1
	ret


;; Main game iteration. Updates all systems and managers. Plays music
man_game_update:
	call sys_input_update
	call sys_ai_update
	call sys_physics_update
	call sys_animation_update 
	call sys_collision_update
	call sys_render_update 

	call man_vecentity_clean_all
	call man_entities_clean

	call update_punctuation
	call itera_oleada

	ld hl, #inmunidad_counter
	xor a 
	cp (hl)
	ret z 
	dec (hl)

	ret



man_game_update_surv:
	call sys_input_update
	call sys_ai_update
	call sys_physics_update
	call sys_animation_update 
	call sys_collision_update
	call sys_render_update 

	call man_vecentity_clean_all
	call man_entities_clean

	call update_punctuation
	call itera_oleada_surv
	ret


;; Prints punctuation and decrements it by time
update_punctuation:
		ld hl, #counter_punctuation
		dec (hl) 
		ret nz
	actualiza:
		push hl  
		
		ld de, #screen_start
		ld c, #screen_width - 12
		ld b, #4
		call cpct_getScreenPtr_asm
		ex de, hl ;; DE ScreenPointer, HL points
		
		push de 
		push bc
		push hl 
		ld e, #1
		ld d, #2
		call cpct_setDrawCharM1_asm
		pop hl 
		pop bc 
		pop de  

		draw_time_points_twice

		pop hl 
		ld (hl), #update_punctuation_wait 
		ld bc, #0x0001
		call dec_points
	ret

;; Will pause/resume the game with K
man_game_check_pause:
	
	ld hl, #Key_K

	call cpct_isKeyPressed_asm
	ret z

	disable_music

	ld a, #25
	check_pause_wait_halts:
	dec a
	halt
	jr nz, check_pause_wait_halts

pausa:
	
	ld hl, #Key_K
	call cpct_isKeyPressed_asm

	jr z, pausa

	ld a, #25
	check_pause_wait_halts_2:
	dec a
	halt
	jr nz, check_pause_wait_halts_2

	enable_music

	ret

;; Wait and press any key to exit
end_waiting:
	

	call cpct_isAnyKeyPressed_f_asm	
	ret nz

	halt

	disable_music

	jr end_waiting

show_punctuation:

    ld l, #video_mem_page_shifted
    call cpct_setVideoMemoryPage_asm

	ld e, #2
	ld d, #0
	call cpct_setDrawCharM1_asm

	ld de, #screen_start
    ld c, #screen_width/2-10
    ld b, #screen_height/2+16
    call cpct_getScreenPtr_asm
    ex de, hl

	ld hl, #score_text
	ld c, #1
	ld b, #0
	call cpct_drawStringM1_f_asm

	ld de, #screen_start
    ld c, #screen_width/2 + 4
    ld b, #screen_height/2 + 16
    call cpct_getScreenPtr_asm
	ex de, hl

    push de 
    push bc
    push hl 
    ld e, #2
	ld d, #0
    call cpct_setDrawCharM1_asm
    pop hl 
    pop bc 
    pop de 

	draw_time_points_twice

	ld d, #end_game_timeout
	ld e, #1
	call wait_halts

	call end_waiting

	ld d, #end_game_timeout
	ld e, #1
	call wait_halts

	ret

man_game_show_gameover:
    ld l, #video_mem_page_shifted
    call cpct_setVideoMemoryPage_asm

	call set_video_mode_big_screen
	call clear_screen_black

	ld hl, #gameover_text
    ld de, #0xC2EE
	ld c, #1
	ld b, #0
	call cpct_drawStringM1_f_asm

	call show_punctuation

	jp man_game_show_title_screen

man_game_show_wingame:
    ld l, #video_mem_page_shifted
    call cpct_setVideoMemoryPage_asm

	call set_video_mode_big_screen
	call clear_screen_black

	ld hl, #wingame_text
    ld de, #0xC2F0
	ld c, #1
	ld b, #0
	call cpct_drawStringM1_f_asm

	call show_punctuation

	jp man_game_show_title_screen


draw_strings:
	ld hl, #tacos_text
    ld de, #0xC014
	ld c, #3
	ld b, #0
	call cpct_drawStringM1_f_asm

	ld hl, #vs_text
    ld de, #0xC027
	ld c, #1
	call cpct_drawStringM1_f_asm

	ld hl, #patos_text
    ld de, #0xC033
	ld c, #2
	call cpct_drawStringM1_f_asm

	ld hl, #chosen_mode
	ld a, (hl)
	and #0xFF
	call nz, draw_selected_second_option

	ld hl, #chosen_mode
	ld a, (hl)
	and #0xFF
	call z, draw_selected_first_option

	ld hl, #text_start_and_pause
    ld de, #0xC46F
	ld c, #1
	ld b, #0
	call cpct_drawStringM1_f_asm

	ld hl, #text_mute
    ld de, #0xC520
	ld c, #1
	ld b, #0
	call cpct_drawStringM1_f_asm

	ld hl, #_g_controles
    ld de, #0xC5BD
	ld b, #50
	ld c, #25
	call cpct_drawSprite_asm

	ret

draw_selected_first_option:
	ld hl, #text_mode_adventure
    ld de, #0xC1F7
	ld bc, #0x0200
	call cpct_drawStringM1_f_asm

	ld hl, #text_mode_survival
    ld de, #0xC338
	ld bc, #0x0002
	call cpct_drawStringM1_f_asm

	ret

draw_selected_second_option:
	ld hl, #text_mode_adventure
    ld de, #0xC1F7
	ld bc, #0x0002
	call cpct_drawStringM1_f_asm

	ld hl, #text_mode_survival
    ld de, #0xC338
	ld bc, #0x0200
	call cpct_drawStringM1_f_asm

	ret

;;Shows the title screen and will loop indefinitely.
;;The loop breaks with ENTER. Game then starts with proper initialization.
man_game_show_title_screen::

	disable_music

    ld l, #video_mem_page_shifted
    call cpct_setVideoMemoryPage_asm
	
	call set_video_mode_big_screen

	call clear_screen_black

	call cpct_waitVSYNC_asm

	call draw_strings

	title_screen_loop:
		
		ld hl, #Key_Space
		call cpct_isKeyPressed_asm
		jr z, check_A
		;; Comprobamos qué modo ha escogido.
		ld hl, #chosen_mode
		ld a, (hl)
		and #0xFF
		jp nz, start_survival
		start_adventure:
			jp man_game_play
			ret
		start_survival:
			jp man_game_play_survival
			ret
		check_A:
			ld hl, #Key_A
			call cpct_isKeyPressed_asm
			jr z, check_Q
			ld hl, #chosen_mode
			ld (hl), #1
			call draw_selected_second_option
			jp title_screen_loop
		check_Q:
			ld hl, #Key_Q
			call cpct_isKeyPressed_asm
			jr z, title_screen_loop
			ld hl, #chosen_mode
			ld (hl), #0
			call draw_selected_first_option
			jp title_screen_loop

;;Sets all video memory to 0
clear_screen_black:
	ld de, #0xC000
	xor a ; a = 0
	ld bc, #0x4000
	call cpct_memset_asm

	ret

;Sets new video mode, new palette and new border
set_video_mode_big_screen:
	ld hl, #big_screen_palette
    ld de, #4
    call cpct_setPalette_asm
	cpctm_setBorder_asm HW_BLACK
	ret

man_game_play_level_surv:
	call man_game_update_surv

	ld hl, #speed_of_game 
	ld d, (hl)
	call wait_halts
	
	call sys_render_switch_buffers
	call man_game_check_pause

	;;call cpct_scanKeyboard_f_asm
	;;ld hl, #Key_Esc
	;;call cpct_isKeyPressed_asm
	;;jp nz, man_game_show_title_screen


	ret


;; Play just one level
;; INPUT: create enemies 
man_game_play_level:
   	call man_game_update

	ld hl, #speed_of_game 
   	ld d, (hl)
	call wait_halts
	
	call sys_render_switch_buffers
   	call man_game_check_pause

   	;;call cpct_scanKeyboard_f_asm
   	;;ld hl, #Key_Esc
   	;;call cpct_isKeyPressed_asm
   	;;jp nz, man_game_show_title_screen

   	ld a, (endgame)
   	or a 
   	jr nz, not_end_game
	ret

not_end_game:
   	ld a, (level_enemies)
   	or a 
   	ret z
   	jr man_game_play_level

;; INPUT a: numero de vidas
;; Asignar las vidas concedidas en el nivel
asigna_vidas:
	ld hl, #vidas_taco
	ld (hl), a
	ret 

;; INPUT C: numero de vidas a sumar
;; Asignar las vidas concedidas en el nivel
suma_vidas:
	ld a, c
	ld hl, #vidas_taco
	add (hl)
	ld b, a 
	ld a, #2
	cp b 
	jr c, demasiadas
	ld (hl), a	
	ret
	demasiadas:
	ld (hl), #3	
	ret 

decrementar_vida_counter:
	ld hl, #vidas_taco
	dec (hl)
	ld hl, #endgame
	ld (hl), #1
	ret


man_game_play_survival:
	call man_game_init
	ld hl, #survive_mode_activated
	ld (hl), #1

	lvlsurv_main:
		disable_music
		ld de, #_music_track_2
		call cpct_akp_musicInit_asm

		ld a, #3
		call asigna_vidas
		jp lvlsurv
	
	jp end_game 

;; Main game loop
man_game_play:
	call man_game_init
	ld hl, #survive_mode_activated
	ld (hl), #0

;; lvl 1, intro
lvl1_main:
	disable_music
	ld de, #_music_track_1
	call cpct_akp_musicInit_asm
	ld a, #2
	call asigna_vidas
	jp lvl1

;; lvl2, 2 fantasmas
lvl2_main:
	pop bc ;; To clean stack
	ld c, #1
	call suma_vidas
	jp lvl2


;; lvl3 presentamos tanques + pajaro
lvl3_main:
	ld c, #1
	call suma_vidas
	jp lvl3

;; lvl4, presenta tempo con patos
lvl4_main:
	disable_music
	ld de, #_music_track_2
	call cpct_akp_musicInit_asm

	pop bc ;; To clean stack

	ld c, #1
	call suma_vidas
	jp lvl4 


;; lvl 5, pájaro + tempo
lvl5_main:
	disable_music
	ld de, #_music_track_1
	call cpct_akp_musicInit_asm

	pop bc ;; To clean stack

	ld c, #1
	call suma_vidas
	jp lvl5


;; lvl 6, 2 fantasmas + 2 patos + ref + tanque + tempo
lvl6_main:
	disable_music
	ld de, #_music_track_2
	call cpct_akp_musicInit_asm

	pop bc ;; To clean stack

	ld c, #3
	call suma_vidas
	jp lvl6

;; lvl7, 
lvl7_main:
	pop bc ;; To clean stack

	jr win_game



win_game:
	ld a, #1
   	ld (endgame), a

	disable_music

   	jp man_game_show_wingame


end_game:


   	ld a, #1
   	ld (endgame), a

	disable_music

	call parpadeo_muere_prota 

   	jp man_game_show_gameover



lvl1:

	ld hl, #actual_lvl
	ld (hl), #1
	call man_lvl_init
    
	ld hl, #max_enemies_on_screen
	ld (hl), #3

	ld a, #0x1F
	ld hl, #oleada_1
	call lanza_oleada

	continua_oleada_lvl1:
	call man_game_play_level

	ld bc, #continua_oleada_lvl1 ;; next oleada
	push bc 
	ld bc, #lvl2_main
	push bc 
	ld a, (endgame)
   	or a 
	jp nz, gestiona_oleadas

	ld bc, #continua_oleada_lvl1 
	push bc
	ld a, (vidas_taco)
   	or a 
	jp nz, pierde_vida
	pop bc 

	jp end_game

lvl2:

	ld hl, #actual_lvl
	ld (hl), #2
	call man_lvl_init

	ld hl, #max_enemies_on_screen
	ld (hl), #4

	ld a, #0x0F
	ld hl, #oleada_2
	call lanza_oleada

	continua_oleada_lvl2:
	call man_game_play_level

	ld bc, #continua_oleada_lvl2 ;; next oleada
	push bc 
	ld bc, #lvl3_main
	push bc 
	ld a, (endgame)
   	or a 
	jp nz, gestiona_oleadas

	ld bc, #continua_oleada_lvl2
	push bc
	ld a, (vidas_taco)
   	or a 
	jp nz, pierde_vida
	pop bc 

	jp end_game


lvl3:

	ld hl, #actual_lvl
	ld (hl), #3
	call man_lvl_init

	ld hl, #max_enemies_on_screen
	ld (hl), #4

	ld a, #0x0F
	ld hl, #oleada_3
	call lanza_oleada

	continua_oleada_lvl3:
	call man_game_play_level

	ld bc, #continua_oleada_lvl3 ;; next oleada
	push bc 
	ld bc, #lvl4_main
	push bc 
	ld a, (endgame)
   	or a 
	jp nz, gestiona_oleadas

	ld bc, #continua_oleada_lvl3
	push bc 
	ld a, (vidas_taco)
   	or a 
	jp nz, pierde_vida
	pop bc 

	jp end_game

lvl4:

	ld hl, #actual_lvl
	ld (hl), #4
	call man_lvl_init

	ld hl, #max_enemies_on_screen
	ld (hl), #5

	ld a, #0x0F
	ld hl, #oleada_4
	call lanza_oleada

	continua_oleada_lvl4:
	call man_game_play_level

	ld bc, #continua_oleada_lvl4 ;; next oleada
	push bc 
	ld bc, #lvl5_main
	push bc 
	ld a, (endgame)
   	or a 
	jp nz, gestiona_oleadas

	ld bc, #continua_oleada_lvl4
	push bc 
	ld a, (vidas_taco)
   	or a 
	jp nz, pierde_vida
	pop bc 

	jp end_game

lvl5:

	ld hl, #actual_lvl
	ld (hl), #5
	call man_lvl_init

	ld hl, #max_enemies_on_screen
	ld (hl), #5

	ld a, #0x0F
	ld hl, #oleada_5
	call lanza_oleada

	continua_oleada_lvl5:
	call man_game_play_level

	ld bc, #continua_oleada_lvl5 ;; next oleada
	push bc 
	ld bc, #lvl6_main
	push bc 
	ld a, (endgame)
   	or a 
	jp nz, gestiona_oleadas

	ld bc, #continua_oleada_lvl5
	push bc 
	ld a, (vidas_taco)
   	or a 
	jp nz, pierde_vida
	pop bc 

	jp end_game

lvl6:

	ld hl, #actual_lvl
	ld (hl), #6
	call man_lvl_init

	ld hl, #max_enemies_on_screen
	ld (hl), #5

	ld a, #0x0F
	ld hl, #oleada_6
	call lanza_oleada


	continua_oleada_lvl6:
	call man_game_play_level

	ld bc, #continua_oleada_lvl6 ;; next oleada
	push bc 
	ld bc, #lvl7_main
	push bc 
	ld a, (endgame)
   	or a 
	jp nz, gestiona_oleadas

	ld bc, #continua_oleada_lvl6
	push bc 
	ld a, (vidas_taco)
   	or a 
	jp nz, pierde_vida
	pop bc 

	jp end_game

pierde_vida: ;; Gain inmunity and continue playing
	call decrementar_vida_counter
	ld bc, #punctuation_losing_life
	call dec_points
	ld hl, #inmunidad_counter
	ld (hl), #inmunidad_time
	draw_vidas_level_twice


	call parpadeo_pierde_vida
	ret


parpadeo_muere_prota:
.globl _g_taco_muerto1
.globl _g_taco_muerto2
.globl _g_taco_muerto3
.globl _g_taco_muerto4
.globl _g_taco_muerto5

	ld hl, #_g_taco_muerto1
	call parpadeo_muere_prota_iteracion

	ld hl, #_g_taco_muerto2
	call parpadeo_muere_prota_iteracion

	ld hl, #_g_taco_muerto3
	call parpadeo_muere_prota_iteracion

	ld hl, #_g_taco_muerto4
	call parpadeo_muere_prota_iteracion

	ld hl, #_g_taco_muerto5
	call parpadeo_muere_prota_iteracion

	ld hl, #taco_empty_sprite
	call parpadeo_muere_prota_iteracion

	ld d, #40
	call wait_halts

	ret 

parpadeo_muere_prota_iteracion:
	ld ix, #_entity_array 
	ld e_sprite_l(ix), l 
	ld e_sprite_h(ix), h 
	call sys_render_update
 
	ld d, #5
	call wait_halts

	ret

parpadeo_pierde_vida:


	ld a, #3
	loop_p:
	push af 
	ld ix, #_entity_array 
	ld l, e_sprite_l(ix) 
	ld h, e_sprite_h(ix) 
	call parpadeo_loop
	pop af 
	dec a 
	jr nz, loop_p

 
	ret 

parpadeo_loop:
	push hl 

	ld ix, #_entity_array 
	ld hl, #taco_empty_sprite
	ld e_sprite_l(ix), l 
	ld e_sprite_h(ix), h 
	 
	call sys_render_update

	ld d, #4
	call wait_halts

	pop hl 
	ld ix, #_entity_array 
	ld e_sprite_l(ix), l 
	ld e_sprite_h(ix), h 
	call sys_render_update	

	ld d, #8
	call wait_halts

	ret



gestiona_oleadas::
	xor a 
	ld hl, #oleada_end
	cp (hl)
	ret z
	pop bc 
	ret 


lvlsurv:

	call man_game_init_lvl_surv

	ld hl, #max_enemies_on_screen
	ld (hl), #4

	ld a, #0x1F
	ld hl, #oleada_1
	call lanza_oleada
	lanza_oleada_surv:

	call man_game_play_level_surv

	ld a, (endgame)
   	or a 
	jp nz, lanza_oleada_surv

	jp end_game
