.module syscol

.include "syscol.h.s"
.include "cpctelera.h.s"
.include "man/collision.h.s"
.include "man/game.h.s"
.include "man/lvl.h.s"
.include "man/entity.h.s"
.include "man/animation.h.s"
.include "assets/template.h.s"
.include "assets/sprites.h.s"
.include "sys/render.h.s"
.include "sys/ai.h.s"

;; Maneja las colisiones. SISTEMA DE COLISIONES

endgame::
    .db #1

;; DESTROY: HL
sys_collision_init::
    call man_collision_getArray_HL
    ld (_entity_array_prt), hl
    ret

.globl _g_explosion_sprite1
.globl _g_explosion_sprite2
.globl _g_explosion_sprite3
.globl _g_explosion_sprite4
.globl _g_explosion_sprite_small1
.globl _g_explosion_sprite_small2
.globl _g_explosion_sprite_small3

animations_death:
	.dw _g_explosion_sprite1  
	.dw _g_explosion_sprite2  
	.dw _g_explosion_sprite3  
    .dw _g_explosion_sprite4  
    .dw nullptr

animations_death_small::
	.dw _g_explosion_sprite_small1  
	.dw _g_explosion_sprite_small2  
	.dw _g_explosion_sprite_small3  
    .dw nullptr

;; Updates all entities
;; Comprobar colisiones con las entidades mayores, ya que el orden no importa y ya han sido comprobadas las combinaciones con menores 
;; Hacer un sort y de esta forma nos ahorramos basto comprobaciones. Lo malo es que solo se puede ordenar en base a un eje.
sys_collision_update::


_entity_array_prt = .+1
    ld hl, #0x0000

loop_ix:
    ;;Get pointer to next entity in DE
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl          ;; HL += 2 (Next pointer)

    ;; si DE = null (final), fuera del bucle grande, se han recorrido todos los ptr1 (y todos los ptr2 con cada uno)
    ld a, e 
    or d 
    ret z

    push hl     ;; siguiente ptr1

    ;; IX = DE
    ld__ixl_e
    ld__ixh_d
    
    push hl
    call check_just_created_ix ;; En tal caso no comprobar colisiones
    pop hl
    jr z, pop_2obucle

loop_iy:
    ;; 2a
    ;;Get pointer to next entity in DE
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl          ;; HL += 2 (Next pointer)

    ;; si DE == null, se acaba el bucle pequeño
    ld a, e 
    or d 
    jp z, pop_2obucle

    ;; IY = DE
    ld__iyl_e
    ld__iyh_d

    push hl
    call check_just_created_iy ;; En tal caso no comprobar colisiones
    pop hl
    jr z, loop_iy

    ;; 1a guardada en IX, 2a giardada en IY
    ;; Comprueba colisiones entre ambas
    call sys_collision_check
    jp c, no_collision
collision:

;; Player taco
    ld a, e_type(ix)
    and #t_player
    call nz, player_behaviour

;; Enemigo
    ld a, #t_enemy|#t_bala_enemy
    and e_type(ix)
    call nz, enemy_behaviour

;; Death animation t_default, no collision

no_collision:
    jp loop_iy

pop_2obucle: ;; Si se pone al principio se hace el pop en la primera iteración, y no queremos eso
    pop hl 
    jp loop_ix



;;Subrutina el elemento iterando es player, comprueba el segundo tipo
player_behaviour:
    ld a, #t_enemy|#t_bala_enemy
    and e_type(iy)
    call nz, muere_player
    ret

;; Subrutina si player choca con enemy
muere_player: 
    push hl 
	ld hl, #inmunidad_counter
    xor a 
    cp (hl)
    pop hl 
    ret nz 

    ;call kill_entity_iy
    ;call kill_entity_ix ;; enemy

    xor a ; a = 0
    ld (endgame), a
    ret

;; Subrutina cuando el iterando es un enemigo
enemy_behaviour:
    ld a, #t_bala
    and e_type(iy)
    call nz, muere_enemy
    ret

;; Subrutina cuando el choque sea entre enemigo y bala
muere_enemy:

    ld a, #t_toBeKilled
    and e_type(iy)    
    ret nz ;; If bullet was killed in previous iteration then jump out

    ;;Special reflective big pato behaviour
    ld a, #et_reflective_pato_big
    cp e_enemy_type(ix)
    jp z, reflective_big_pato_behaviour

    ld a, #t_invisible
    and e_type(ix)    
    ret nz ;; Invisible entities are not affected by bullets hihi

    call kill_entity_iy
    call bala_to_enemy_behaviour

    ret

;; IX = enemy
;; Will only call ONE behaviour, depending on enemy type
bala_to_enemy_behaviour:
    ld bc, #end_bala_to_enemy_behaviour
    push bc

    ld a, e_enemy_type(ix)

    cp #et_pato
    jp z, pato_behaviour

    cp #et_pato_small
    jp z, pato_small_behaviour

    cp #et_tempo
    jp z, tempo_behaviour
    
    cp #et_reflective_pato_small
    jp z, pato_reflective_small_behaviour

    cp #et_fantasma
    jp z, fantasma_behaviour
    
    cp #et_tanque
    jp z, tanque_behaviour 
    
    pop bc 
    end_bala_to_enemy_behaviour:
    ret


;; Subrutina enemigo muerto es pato
pato_behaviour:
    call kill_entity_ix

    push iy
    ld iy, #pato_small_entidad
    ld a, e_x0(ix)
    inc a
	ld e_x0(iy), a
    ld a, e_y0(ix) 
	ld e_y0(iy), a

    ld bc, #behaviour_pato_otro_eje
    ld e_ai_behaviour_l(iy), c
    ld e_ai_behaviour_h(iy), b
    ld e_vx(iy), #0
    ld e_vy(iy), #1
    ld e_estado_ia(iy), #sinusoidal_pato
    pop iy
    ld hl, #pato_small_entidad
	call man_entity_create_save_ix

    push iy
    ld iy, #pato_small_entidad
    ld a, e_x0(ix)
	ld e_x0(iy), a
    ld a, e_y0(ix) 
	ld e_y0(iy), a

    ld bc, #behaviour_pato
    ld e_ai_behaviour_l(iy), c
    ld e_ai_behaviour_h(iy), b
    ld a, e_vx(ix)
    ld e_vx(iy), a
    ld e_vy(iy), #0
    ld e_estado_ia(iy), #basic_pato_down_state
    pop iy
    ld hl, #pato_small_entidad
	call man_entity_create_save_ix

    push bc 
	ld bc, #punctuation_pato
	push hl
    call add_points
    pop hl
    pop bc 

    ret


pato_reflective_small_behaviour:
    push hl
    ld hl, #small_fast_pato_lifes
    dec (hl)

    xor a 
    ld (reflective_patos), a ;; must be only one on screen
    pop hl
    ret nz

    ;call kill_entity_ix
    push hl 
    ld hl, #animations_death
    call kill_entity_ix_animation_death
    pop hl 

    push bc 
	push hl

	ld bc, #punctuation_reflective_pato
    call add_points

    pop hl
    pop bc 
    
    ret
    

;; Subrutina enemigo muerto es pato small
pato_small_behaviour::
    ;;call kill_entity_ix

    push hl 
    ld hl, #animations_death_small
    call kill_entity_ix_animation_death
    pop hl 

    push bc 
	ld bc, #punctuation_pato_small
	push hl
    call add_points
    pop hl
    pop bc 
    

    ret

;; Subrutina enemigo muerto es tempo
tempo_behaviour:
    ; push hl CHECK

    ld hl, #main_palette
    ld de, #4
    call cpct_setPalette_asm

    ld hl, #speed_of_game
    ld (hl), #speed_standard ;; Reseteo de cambios en velocidad

    ld hl, #tempo_vidas_restantes
    dec (hl)
    jp nz, quita_una_vida
    ld (hl), #vidas_tempo
    
    ;call kill_entity_ix
    push hl 
    ld hl, #animations_death
    call kill_entity_ix_animation_death
    pop hl 

    push bc 
	ld bc, #punctuation_tempo
	push hl
    call add_points
    pop hl
    pop bc 
    

    ret
    
    quita_una_vida: 
    ; pop hl CHECK
    ret

;; INPUT: IY (bullet)
;; Makes the bullet go in oposite direction
;; If bullet is pointing up then vy is negated
;; otherwise, vx is negated
;; Also bullet TURNS INTO enemy + bullet type (so it will kill player if it hits him)
;; Bullet's orientation remains unchanged
;; Bullet's X and Y calculations for this frame ARE REVERTED
reflective_big_pato_behaviour:

    ld e_type(iy), #t_enemy|t_bala

    ;;Check orientation != up
    ld a, #o_up
    cp e_orientation(iy)
    jr nz, bullet_coming_from_side

    ;;Bullet is pointing up
    ld a, e_vy(iy)
    ld b, e_y0(iy)
    neg
    ld e_vy(iy), a
    add a, b
    ld e_y0(iy), a
    ret

    ;;Bullet is pointing left or right
    bullet_coming_from_side:
    ld a, e_vx(iy)
    ld b, e_x0(iy)
    neg
    ld e_vx(iy), a
    add a, b
    ld e_x0(iy), a
    ret

;; Subrutina enemigo muerto es tanque
tanque_behaviour:
    ;call kill_entity_ix
    push hl 
    ld hl, #animations_death
    call kill_entity_ix_animation_death
    pop hl 

    push bc 
	ld bc, #punctuation_tanque
	push hl
    call add_points
    pop hl
    pop bc 
    
    ret

;; Subrutina enemigo muerto es fantasma
fantasma_behaviour:
    ;call kill_entity_ix
    push hl 
    ld hl, #animations_death
    call kill_entity_ix_animation_death
    pop hl 

    push bc 
	ld bc, #punctuation_fantasma
	push hl
    call add_points
    pop hl
    pop bc 
    
    ret

;; INPUT: IX: entity HL: animation_array
kill_entity_ix_animation_death:
    ld e_animated_l(ix), l 
    ld e_animated_h(ix), h 

    ld hl, #behaviour_muerto_por_bala
    ld e_ai_behaviour_l(ix), l 
    ld e_ai_behaviour_h(ix), h

    ld e_vx(ix), #0
    ld e_vy(ix), #0
    
    ld e_type(ix), #t_default
    ret

;; Mata una entidad asignandole el bit tobekilled, se decrementa el counter solo con ix porque los enemigos solo pueden morir iterando ellos
;; INPUT: IX
kill_entity_ix::
	ld a, #t_enemy
    and e_type(ix)
	jp z, no_es_enemy_ix
kill_entity_ix_not_enemy::
    push hl
    ld hl, #level_enemies ;; Decrementar el counter del nivel
	dec (hl)
    pop hl
no_es_enemy_ix:
    ld a, e_type(ix)
    or #t_toBeKilled
    ld e_type(ix), a

    ret

;; Mata una entidad asignandole el bit tobekilled
;; INPUT: IY
kill_entity_iy:
    ld a, e_type(iy)
    or #t_toBeKilled
    ld e_type(iy), a

    ret

;; IX entity_1
;; IY entity_2
;; DESTROY: AF
;; Return carry: No colisiona. no carry SI colisiona
sys_collision_check::

    ld a, e_x0(ix)
    add e_w(ix)
    sub e_x0(iy)
    ret c

    ld a, e_x0(iy)
    add e_w(iy)
    sub e_x0(ix)
    ret c

;;;;EJE Y
    ;; Por debajo
    ld a, e_y0(ix)
    add e_h(ix)
    sub e_y0(iy)
    ret c

    ;; Por arriba
    ld a, e_y0(iy)
    add e_h(iy)
    sub e_y0(ix)

    ret
